@@ -2,8 +2,7 @@
 Export version 1 JSON schema (separate meta and tree JSONs) for visualization with Auspice
 """
 
-import os, sys
-import re
+import sys
 from textwrap import dedent
 import time
 import numpy as np
@@ -11,7 +10,6 @@ from Bio import Phylo
 from argparse import SUPPRESS
 from collections import defaultdict
 from .errors import AugurError
-from .argparse_ import ExtendAction
 from .io.metadata import DEFAULT_DELIMITERS, InvalidDelimiter, read_metadata
 from .utils import read_node_data, write_json, read_config, read_lat_longs, read_colors
 
@@ -316,7 +314,7 @@ def add_core_args(parser):
     core.add_argument('--metadata', required=True, metavar="FILE", help="sequence metadata")
     core.add_argument('--metadata-delimiters', default=DEFAULT_DELIMITERS, nargs="+",
                       help="delimiters to accept when reading a metadata file. Only one delimiter will be inferred.")
-    core.add_argument('--node-data', required=True, nargs='+', action=ExtendAction, help="JSON files with meta data for each node")
+    core.add_argument('--node-data', required=True, nargs='+', action="extend", help="JSON files with meta data for each node")
     core.add_argument('--output-tree', help="JSON file name that is passed on to auspice (e.g., zika_tree.json).")
     core.add_argument('--output-meta', help="JSON file name that is passed on to auspice (e.g., zika_meta.json).")
     core.add_argument('--auspice-config', help="file with auspice configuration")
