@@ -10,7 +10,8 @@ from .errors import AugurError
 from .frequency_estimators import get_pivots, alignment_frequencies, tree_frequencies
 from .frequency_estimators import AlignmentKdeFrequencies, TreeKdeFrequencies, TreeKdeFrequenciesError
 from .dates import numeric_date_type, SUPPORTED_DATE_HELP_TEXT, get_numerical_dates
-from .io.metadata import DEFAULT_DELIMITERS, DEFAULT_ID_COLUMNS, InvalidDelimiter, read_metadata
+from .io.file import open_file
+from .io.metadata import DEFAULT_DELIMITERS, DEFAULT_ID_COLUMNS, METADATA_DATE_COLUMN, InvalidDelimiter, Metadata, read_metadata
 from .utils import write_json
 
 
@@ -85,27 +86,38 @@ def format_frequencies(freq):
 
 def run(args):
     try:
-        metadata = read_metadata(args.metadata, delimiters=args.metadata_delimiters, id_columns=args.metadata_id_columns)
+        metadata_object = Metadata(args.metadata, args.metadata_delimiters, args.metadata_id_columns)
     except InvalidDelimiter:
         raise AugurError(
             f"Could not determine the delimiter of {args.metadata!r}. "
             f"Valid delimiters are: {args.metadata_delimiters!r}. "
             "This can be changed with --metadata-delimiters."
         )
+
+    columns_to_load = [metadata_object.id_column, METADATA_DATE_COLUMN]
+    if args.weights_attribute:
+        columns_to_load.append(args.weights_attribute)
+    metadata = read_metadata(
+        args.metadata,
+        delimiters=[metadata_object.delimiter],
+        columns=columns_to_load,
+        id_columns=[metadata_object.id_column],
+        dtype="string",
+    )
     dates = get_numerical_dates(metadata, fmt='%Y-%m-%d')
     stiffness = args.stiffness
     inertia = args.inertia
 
-    if args.method == "kde":
-        # Load weights if they have been provided.
-        if args.weights:
-            with open(args.weights, "r", encoding='utf-8') as fh:
-                weights = json.load(fh)
+    # For KDE
+    weights = None
+    weights_attribute = None
 
-            weights_attribute = args.weights_attribute
-        else:
-            weights = None
-            weights_attribute = None
+    if args.method == "kde" and args.weights:
+        # Load weights if they have been provided.
+        with open_file(args.weights, "r") as fh:
+            weights = json.load(fh)
+
+        weights_attribute = args.weights_attribute
 
     if args.tree:
         tree = Phylo.read(args.tree, 'newick')
@@ -114,10 +126,9 @@ def run(args):
             tip.attr = {"num_date": np.mean(dates[tip.name])}
             tps.append(tip.attr["num_date"])
 
-            # Annotate tips with metadata to enable filtering and weighting of
-            # frequencies by metadata attributes.
-            for key, value in metadata.loc[tip.name].items():
-                tip.attr[key] = value
+            if weights_attribute:
+                # Annotate tip with weight attribute.
+                tip.attr[weights_attribute] = metadata.loc[tip.name, weights_attribute]
 
         if args.method == "diffusion":
             # estimate tree frequencies
