@@ -6,7 +6,7 @@ import sys
 from Bio import Phylo
 from .dates import get_numerical_dates
 from .dates.errors import InvalidYearBounds
-from .io.metadata import DEFAULT_DELIMITERS, DEFAULT_ID_COLUMNS, InvalidDelimiter, read_metadata
+from .io.metadata import DEFAULT_DELIMITERS, DEFAULT_ID_COLUMNS, METADATA_DATE_COLUMN, InvalidDelimiter, Metadata, read_metadata
 from .utils import read_tree, write_json, InvalidTreeError
 from .errors import AugurError
 from treetime.vcf_utils import read_vcf
@@ -213,17 +213,24 @@ def run(args):
         if args.metadata is None:
             print("ERROR: meta data with dates is required for time tree reconstruction", file=sys.stderr)
             return 1
+
         try:
-            metadata = read_metadata(
-                args.metadata,
-                delimiters=args.metadata_delimiters,
-                id_columns=args.metadata_id_columns)
+            metadata_object = Metadata(args.metadata, args.metadata_delimiters, args.metadata_id_columns)
         except InvalidDelimiter:
             raise AugurError(
                 f"Could not determine the delimiter of {args.metadata!r}. "
                 f"Valid delimiters are: {args.metadata_delimiters!r}. "
                 "This can be changed with --metadata-delimiters."
             )
+
+        metadata = read_metadata(
+            args.metadata,
+            delimiters=[metadata_object.delimiter],
+            columns=[metadata_object.id_column, METADATA_DATE_COLUMN],
+            id_columns=[metadata_object.id_column],
+            dtype="string",
+        )
+
         try:
             dates = get_numerical_dates(metadata, fmt=args.date_format,
                                         min_max_year=args.year_bounds)
@@ -232,8 +239,8 @@ def run(args):
 
         # save input state string for later export
         for n in T.get_terminals():
-            if n.name in metadata.index and 'date' in metadata.columns:
-                n.raw_date = metadata.at[n.name, 'date']
+            if n.name in metadata.index and METADATA_DATE_COLUMN in metadata.columns:
+                n.raw_date = metadata.at[n.name, METADATA_DATE_COLUMN]
 
         if args.date_confidence:
             time_inference_mode = 'always' if args.date_inference=='marginal' else 'only-final'
