@@ -15,6 +15,7 @@ if __name__ == "__main__":
     parser.add_argument("--significant-digits", type=int, default=5, help="number of significant digits to use when comparing numeric values")
     parser.add_argument("--exclude-paths", nargs="+", help="list of paths to exclude from consideration when performing a diff", default=["root['generated_by']['version']"])
     parser.add_argument("--exclude-regex-paths", nargs="+", help="list of path regular expressions to exclude from consideration when performing a diff")
+    parser.add_argument("--ignore-numeric-type-changes", action="store_true", help="ignore numeric type changes in the diff (e.g., int of 1 to float of 1.0)")
 
     args = parser.parse_args()
 
@@ -31,5 +32,6 @@ if __name__ == "__main__":
             significant_digits=args.significant_digits,
             exclude_paths=args.exclude_paths,
             exclude_regex_paths=args.exclude_regex_paths,
+            ignore_numeric_type_changes=args.ignore_numeric_type_changes,
         )
     )
